package com.muyu.task.server.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.muyu.common.core.domain.Result;
import com.muyu.common.domian.TaskInfo;
import com.muyu.common.domian.TaskInput;
import com.muyu.common.domian.req.TaskInfoAddReq;
import com.muyu.common.domian.req.TaskInfoListReq;
import com.muyu.common.domian.req.TaskInfoUpdReq;
import com.muyu.common.domian.resp.TaskInfoResp;
import com.muyu.task.server.service.NodeJointService;
import com.muyu.task.server.service.NodeTableService;
import com.muyu.task.server.service.TaskInfoService;
import com.muyu.task.server.service.TaskInputService;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.CollectionUtils;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import java.util.HashMap;
import java.util.List;

/**
 * @author Administrator
 */
@RestController
@RequestMapping("taskInfo")
@Tag(name = "任务控制层", description = "进行任务管理查询操作")
public class TaskInfoController {

    @Autowired
    private TaskInfoService taskInfoService;

    @Autowired
    private TaskInputService taskInputService;

    @Autowired
    private NodeJointService nodeJointService;
    /**
     * 获取所有的任务信息
     * @param req
     * @return
     */

    @PostMapping("/list")
    @Operation(summary = "查看任务", description = "根据任务名称 权重 状态 查询")
    public Result<Page<TaskInfoResp>> selectList(@RequestBody TaskInfoListReq req) {
        return Result.success(taskInfoService.selectList(req));
    }

    /**
     * 添加任务信息
     * @param req
     * @return
     */

    @PostMapping()
    @Operation(summary = "添加任务", description = "添加任务信息表")
    public Result save(@RequestBody @Validated TaskInfoAddReq req) {
        taskInfoService.save(TaskInfo.addBuild(req));
        return Result.success();
    }

    /**
     * 通过id查询任务信息
     * @param id
     * @return
     */

    @GetMapping("/{id}")
    @Operation(summary = "任务信息查询",description = "通过任务id查询任务信息")
    public Result<TaskInfoResp> selectById(@PathVariable("id") Long id) {
        TaskInfo byId = taskInfoService.getById(id);
        return Result.success(TaskInfo.build(byId));
    }


    /**
     * 通过id删除任务
     * @param id
     * @return
     */
    @DeleteMapping("/{id}")
    @Operation(summary = "删除任务",description = "通过任务id删除任务信息")
    public Result delete(@PathVariable("id") Long id) {
        taskInfoService.removeById(id);
        return Result.success(null,"操作成功");
    }

    /**
     * 通过id修改任务
     * @param id
     * @param req
     * @return
     */
    @PutMapping("/{id}")
    @Operation(summary = "任务信息修改",description = "通过任务id修改任务信息")
    public Result update(@PathVariable("id") Long id , @RequestBody TaskInfoUpdReq req){
        taskInfoService.updateById(TaskInfo.updBuild(req,()->id));
        return Result.success(null,"操作成功");
    }



    @PostMapping("/updateByJson/{id}")
    @Operation(summary = "修改任务Json", description = "修改任务Json")
    public Result updateByJson(@PathVariable("id") Long id ,@RequestBody String json) {
        UpdateWrapper<TaskInfo> wrapper = new UpdateWrapper<>();
        wrapper.eq("id",id);
        wrapper.set("json",json);
        taskInfoService.update(wrapper);
        return Result.success(null,"保存成功");
    }

    @PostMapping("/execute")
    public Result execute(@RequestParam("taskId") Long taskId){
        taskInfoService.findByFieName2(taskId);
        return Result.success();
    }




}
