package com.muyu.task.server.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.muyu.common.core.domain.Result;
import com.muyu.common.domian.TaskInput;
import com.muyu.common.domian.req.InputAddReq;
import com.muyu.common.domian.req.TaskInputListReq;
import com.muyu.common.domian.req.TaskInputUpdReq;
import com.muyu.common.domian.resp.TaskInputResp;
import com.muyu.common.domian.vo.TableVo;
import com.muyu.task.server.service.TaskInputService;
import io.swagger.v3.oas.annotations.Operation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.HashMap;
import java.util.List;

/**
 * @author Administrator
 */
@RestController
@RequestMapping("TaskInput")
public class TaskInputController {

    @Autowired
    private TaskInputService taskInputService;

    /**
     * 获取所有的任务信息
     *
     * @param req
     * @return
     */
    @PostMapping("list")
    @Operation(summary = "查看节点", description = "根据节点名称 权重 状态 查询")
    public Result list(@RequestBody TaskInputListReq req) {
        Page<TaskInput> list = taskInputService.findList(req);
        return Result.success(list);
    }

    /**
     * 添加节点信息
     *
     * @param req
     * @return
     */
    @PostMapping()
    @Operation(summary = "添加节点", description = "添加任务信息表")
    public Result save(@RequestBody InputAddReq req) {
        HashMap<String, Object> map = new HashMap<>();
        map.put("node_id",req.getNodeId());
        taskInputService.removeByMap(map);
        List<TableVo> list = req.getList();
        String tableFieId="";
        String tableAsFieId="";
        String fieIdAsEngineId="";
        for (TableVo tableVo : list) {
            tableFieId += "," + tableVo.getColumnName();
            tableAsFieId += "," + tableVo.getTableAsName();
            fieIdAsEngineId += "," + tableVo.getFieIdAsEngineId();
        }
        tableFieId = tableFieId.substring(1);
        tableAsFieId = tableAsFieId.substring(1);
        fieIdAsEngineId = fieIdAsEngineId.substring(1);

//        List<TaskInput> inputs = list.stream().map(inputAddReq -> {
//            TaskInput build = TaskInput.builder()
//                    .nodeId(req.getNodeId())
//                    .nodeName(req.getNodeName())
//                    .taskId(req.getTaskId())
//                    .databaseId(req.getDatabaseId())
//                    .tableName(req.getTableName())
//                    .tableAsName(req.getTableAsName())
//                    .tableFieId(inputAddReq.getColumnName())
//                    .tableAsFieId(inputAddReq.getTableAsName())
//                    .fieIdAsEngineId(inputAddReq.getFieIdAsEngineId())
//                    .build();
//            return build;
//        }).toList();
        TaskInput build = TaskInput.builder()
                    .nodeId(req.getNodeId())
                    .nodeName(req.getNodeName())
                    .taskId(req.getTaskId())
                    .databaseId(req.getDatabaseId())
                    .tableName(req.getTableName())
                    .tableAsName(req.getTableAsName())
                    .tableFieId(tableFieId)
                    .tableAsFieId(tableAsFieId)
                    .fieIdAsEngineId(fieIdAsEngineId)
                    .build();
        taskInputService.save(build);
        return Result.success();
    }

    /**
     * 通过id查询任务信息
     *
     * @param id
     * @return
     */
    @GetMapping("/{id}")
    @Operation(summary = "任务信息查询", description = "通过任务id查询任务信息")
    public Result selectById(@PathVariable("id") Long id) {
        QueryWrapper<TaskInput> wrapper = new QueryWrapper<>();
        wrapper.eq("node_id",id);
        TaskInput one = taskInputService.getOne(wrapper);
        TaskInputResp build = TaskInput.build(one);
        return Result.success(build);
    }

    /**
     * 通过id删除任务
     *
     * @param id
     * @return
     */
    @DeleteMapping("/{id}")
    @Operation(summary = "删除任务", description = "通过任务id删除任务信息")
    public Result delete(@PathVariable("id") Long id) {
        QueryWrapper<TaskInput> taskInputQueryWrapper = new QueryWrapper<>();
        taskInputQueryWrapper.eq("node_id",id);
        taskInputService.remove(taskInputQueryWrapper);
        return Result.success(null, "操作成功");
    }

    /**
     * 通过id修改任务
     *
     * @param id
     * @param req
     * @return
     */

    @PutMapping("/{id}")
    @Operation(summary = "任务信息修改", description = "通过任务id修改任务信息")
    public Result update(@PathVariable("id") Integer id, @RequestBody TaskInputUpdReq req) {
        taskInputService.updateById(TaskInput.updBuild(req, () -> id));
        return Result.success(null, "操作成功");
    }
    @PostMapping("/findByTableFieName/{taskId}")
    public Result findByTableFieName(@PathVariable("taskId") Long taskId){
        QueryWrapper<TaskInput> wrapper = new QueryWrapper<>();
        wrapper.eq("task_id",taskId);
        List<TaskInput> list = taskInputService.list(wrapper);
        return Result.success(list);
    }
}
