package com.muyu.task.server.service.impl;

import com.muyu.task.server.thread.OptimizedPrioritizedThreadPool;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.muyu.common.core.domain.Result;
import com.muyu.common.core.utils.StringUtils;
import com.muyu.common.domain.DataValue;
import com.muyu.common.domian.NodeJoint;
import com.muyu.common.domian.TaskInfo;
import com.muyu.common.domian.TaskInput;
import com.muyu.common.domian.TaskOutput;
import com.muyu.common.domian.enums.Weight;
import com.muyu.common.domian.req.TaskInfoListReq;
import com.muyu.common.domian.resp.TaskInfoResp;
import com.muyu.remote.feign.DatasourceFeign;
import com.muyu.task.server.mapper.TaskInfoMapper;
import com.muyu.task.server.service.NodeJointService;
import com.muyu.task.server.service.TaskInfoService;
import com.muyu.task.server.service.TaskInputService;
import com.muyu.task.server.service.TaskOutputService;
import lombok.extern.log4j.Log4j2;
import org.jetbrains.annotations.NotNull;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.util.*;
import java.util.concurrent.*;

import static com.muyu.task.server.thread.OptimizedPrioritizedThreadPool.*;

/**
 * @author Administrator
 */
@Log4j2
@Service
public class TaskInfoServiceImpl extends ServiceImpl<TaskInfoMapper, TaskInfo> implements TaskInfoService {

    private final Integer PAGE_SIZE = 50000;

    @Resource
    private TaskInfoMapper taskInfoMapper;
    @Autowired
    private TaskInputService taskInputService;

    @Autowired
    private NodeJointService nodeJointService;

    @Autowired
    private TaskOutputService taskOutputService;


    @Autowired
    private DatasourceFeign datasourceFeign;

    @Override
    public Page<TaskInfoResp> selectList(TaskInfoListReq req) {
        LambdaQueryWrapper<TaskInfo> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.like(StringUtils.isNotEmpty(req.getName()), TaskInfo::getName, req.getName());
        queryWrapper.eq(req.getWeight() != null, TaskInfo::getWeight, req.getWeight());
        queryWrapper.eq(StringUtils.isNotEmpty(req.getProcessStatus()), TaskInfo::getProcessStatus, req.getProcessStatus());
        Page<TaskInfo> page = new Page<>(req.getPageNum(), req.getPageSize());
        Page<TaskInfo> taskInfoPage = taskInfoMapper.selectPage(page, queryWrapper);
        long pages = taskInfoPage.getPages();
        long size = taskInfoPage.getSize();
        long current = taskInfoPage.getCurrent();
        List<TaskInfoResp> list = taskInfoPage.getRecords().stream().map(taskInfo -> TaskInfo.build(taskInfo)).toList();
        Page<TaskInfoResp> respPage = new Page<>();
        respPage.setRecords(list);
        respPage.setCurrent(current);
        respPage.setSize(size);
        respPage.setTotal(this.count(queryWrapper));
        respPage.setPages(pages);
        return respPage;
    }

//    @Override
//    public String findByFieName(Long taskId) {
//        QueryWrapper<TaskInput> wrapper = new QueryWrapper<>();
//        wrapper.eq("task_id", taskId);
//        HashSet<Long> longs = new HashSet<>();
//        List<TaskInput> list = taskInputService.list(wrapper);
//        if (CollectionUtils.isEmpty(list)) {
//            return "没有选择字段";
//        }
//        String fieName = "";
//        HashMap<String, Long> hashMap = new HashMap<>();
//        for (int i = 0; i < list.size(); i++) {
//            fieName += "," + list.get(i).getTableAsFieId();
//            Long databaseId = list.get(i).getDatabaseId();
//            longs.add(databaseId);
//            String tableFieId = list.get(i).getTableFieId();
//            Long fieIdAsEngineId = list.get(i).getFieIdAsEngineId();
//            hashMap.put(tableFieId,fieIdAsEngineId);
//        }
//        if (longs.size() > 1) {
//            return "你选择的不是同一个数据库";
//        }
//
//        QueryWrapper<NodeJoint> jointQueryWrapper = new QueryWrapper<>();
//        jointQueryWrapper.eq("task_id", taskId);
//        List<NodeJoint> jointList = nodeJointService.list(jointQueryWrapper);
//        String joint = "";
//        if (!CollectionUtils.isEmpty(jointList)) {
//            for (NodeJoint nodeJoint : jointList) {
//                String oneNodeId = nodeJoint.getOneNodeId();
//                TaskInput taskInput = taskInputService.selectByNodeId(oneNodeId);
//                String tableName = taskInput.getTableName();
//                String tableAsName = taskInput.getTableAsName();
//
//                String twoNodeId = nodeJoint.getTwoNodeId();
//                TaskInput taskInputTwo = taskInputService.selectByNodeId(twoNodeId);
//                String tableNameTwo = taskInputTwo.getTableName();
//                String tableAsNameTwo = taskInputTwo.getTableAsName();
//                joint += " " + tableName + " " + " " + tableAsName + " " + nodeJoint.getJoint()
//                        + " " + tableNameTwo + " " + " " + tableAsNameTwo + " on " + nodeJoint.getOneFie()
//                        + " = " + nodeJoint.getTwoFie();
//
//            }
//        }else {
//            TaskInput taskInput = taskInputService.selectByTaskId(taskId);
//            String tableName = taskInput.getTableName();
//            String tableAsName = taskInput.getTableAsName();
//            joint=" "+ tableName+" "+tableAsName;
//        }
//        Long basicId = taskInputService.selectByBasicId(taskId);
//        fieName = fieName.substring(1);
//        String sql = " SELECT " + fieName + " FROM "+ joint;
//        System.out.println(sql);
//        Result<List<List<DataValue>>> tableValueResult = datasourceFeign.findTableValue(basicId, sql);
//        QueryWrapper<TaskOutput> queryWrapper = new QueryWrapper<>();
//        queryWrapper.eq("task_id", taskId);
//        List<TaskOutput> list1 = taskOutputService.list(queryWrapper);
//        HashMap<String, String> map = new HashMap<>();
//        Long tableId=null;
//        Long newBasicId=null;
//        for (TaskOutput taskOutput : list1) {
//            tableId=taskOutput.getTableId();
//            newBasicId=taskOutput.getBasicId();
//            String lastFieName = taskOutput.getLastFieName();
//            String newFieName = taskOutput.getNewFieName();
//            map.put(lastFieName, newFieName);
//        }
//        List<List<DataValue>> tableValue = tableValueResult.getData();
//        for (List<DataValue> dataValues : tableValue) {
//            for (DataValue dataValue : dataValues) {
//                String key = dataValue.getKey();
//                String newKey = map.get(key);
//                dataValue.setKey(newKey);
//            }
//        }
//        return "执行成功";
//    }

    @Override
    public String findByFieName2(Long taskId) {
        long start = System.currentTimeMillis();
        TaskInfo taskInfo = taskInfoMapper.selectById(taskId);
        String weight = taskInfo.getWeight();
        QueryWrapper<TaskInput> wrapper = new QueryWrapper<>();
        wrapper.eq("task_id", taskId);
        List<TaskInput> list = taskInputService.list(wrapper);
        if (CollectionUtils.isEmpty(list)) {
            return "没有选择表";
        }
        String fieName = "";

        Map<String, Object> fie = getFieAsNameAndFieEngineId(list);
        Map<String, String> fieAsName = (Map<String, String>) fie.get("FieAsName");
        Map<String, String> fieEngineId = (Map<String, String>) fie.get("FieEngineId");
        Set<Long> longs = (Set<Long>) fie.get("longs");
        if (longs.size() > 1) {
            return "你选择的不是同一个数据库";
        }

        //联查的方法
        String joint = getJoint(taskId);


        Long basicId = taskInputService.selectByBasicId(taskId);
        QueryWrapper<TaskOutput> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("task_id", taskId);
        TaskOutput one = taskOutputService.getOne(queryWrapper);
        if (one == null) {
            return "没有选择输出";
        }
        Long tableId = one.getTableId();
        Long newBasicId = one.getBasicId();
        HashMap<String, String> map = new HashMap<>();
        String[] newFieName = one.getNewFieName().split(",");
        String[] lastFieName = one.getLastFieName().split(",");
        Integer num = lastFieName.length;
        for (int i = 0; i < newFieName.length; i++) {
            map.put(lastFieName[i], newFieName[i]);
            fieName += "," + fieAsName.get(lastFieName[i]);
        }
        fieName = fieName.substring(1);
        //查询表的数量
        Long count = getCount(joint, basicId);
        //查询和添加
        extracted(count, weight, fieName, joint, basicId, newBasicId, tableId, map, num);
        long end = System.currentTimeMillis();
        //log.info("执行时间：{}",end-start);
        return null;
    }

    private Long getCount(String joint, Long basicId) {
        String sql = " SELECT count(1) FROM " + joint;
        System.out.println(sql);
        Result<Long> countResult = datasourceFeign.findCount(basicId, sql);
        Long data = countResult.getData();
        return data;
    }

    private void extracted(Long data, String weight, String finalFieName, String finalJoint, Long basicId, Long newBasicId, Long tableId, HashMap<String, String> map, Integer num) {
        long count = data / PAGE_SIZE + (data % PAGE_SIZE > 0 ? 1 : 0);
        for (long i = 1; i <= count; i++) {
            long pageNum = (i - 1) * PAGE_SIZE;
            long pageSize = Math.min(PAGE_SIZE, data - pageNum);
            if (Weight.high.getValue().equals(weight)) {
                log.info("执行高级任务");
                submitHighPriorityTask(() -> {
                    getString(pageNum, finalFieName, finalJoint, basicId, newBasicId, tableId, map, pageSize, num);
                });

            }
            if (Weight.centre.getValue().equals(weight)) {
                submitMediumPriorityTask(() -> {
                    getString(pageNum, finalFieName, finalJoint, basicId, newBasicId, tableId, map, pageSize, num);
                });
            }
            if (Weight.low.getValue().equals(weight)) {

                submitLowPriorityTask(() -> {
                    getString(pageNum, finalFieName, finalJoint, basicId, newBasicId, tableId, map, pageSize, num);
                });

            }
            if (Weight.urgency.getValue().equals(weight)) {
                log.info("执行紧急任务");
                // 调整线程分配以适应紧急任务
                OptimizedPrioritizedThreadPool.activeEmergencyTasks.set(0);
                OptimizedPrioritizedThreadPool.remainingTasks.set(0);
                submitEmergencyTask(() -> {
                    try {
                        getString(pageNum, finalFieName, finalJoint, basicId, newBasicId, tableId, map, pageSize, num);
                    } finally {
                        // 减少剩余任务计数
                        if (OptimizedPrioritizedThreadPool.remainingTasks.decrementAndGet() == 0) {
                            System.out.println("All emergency tasks have completed.");
                        }
                    }
                });


            }
        }

    }


    @NotNull
    private Map<String, Object> getFieAsNameAndFieEngineId(List<TaskInput> list) {
        HashMap<String, String> tableNameMap = new HashMap<>();
        HashSet<Long> longs = new HashSet<>();
        HashMap<String, String> hashMap = new HashMap<>();
        for (int i = 0; i < list.size(); i++) {
            Long databaseId = list.get(i).getDatabaseId();
            longs.add(databaseId);
            String[] tableFieId = list.get(i).getTableFieId().split(",");
            String[] tableAsFieId = list.get(i).getTableAsFieId().split(",");
            String[] fieIdAsEngineId = list.get(i).getFieIdAsEngineId().split(",");
            for (int j = 0; j < tableFieId.length; j++) {
                hashMap.put(tableFieId[j], fieIdAsEngineId[j]);
                tableNameMap.put(tableFieId[j], tableAsFieId[j]);
            }
        }
        HashMap<String, Object> map = new HashMap<>();
        map.put("FieAsName", tableNameMap);
        map.put("FieEngineId", hashMap);
        map.put("longs", longs);
        return map;
    }


    @NotNull
    private String getJoint(Long taskId) {
        String joint = "";
        QueryWrapper<NodeJoint> jointQueryWrapper = new QueryWrapper<>();
        jointQueryWrapper.eq("task_id", taskId);
        List<NodeJoint> jointList = nodeJointService.list(jointQueryWrapper);
        if (!CollectionUtils.isEmpty(jointList)) {
            for (NodeJoint nodeJoint : jointList) {
                String oneNodeId = nodeJoint.getOneNodeId();
                QueryWrapper<TaskInput> queryWrapper = new QueryWrapper<>();
                queryWrapper.eq("node_id", oneNodeId);
                TaskInput taskInput = taskInputService.getOne(queryWrapper);
                String tableName = taskInput.getTableName();
                String tableAsName = taskInput.getTableAsName();

                String twoNodeId = nodeJoint.getTwoNodeId();
                QueryWrapper<TaskInput> queryWrapperTwo = new QueryWrapper<>();
                queryWrapperTwo.eq("node_id", twoNodeId);
                TaskInput taskInputTwo = taskInputService.getOne(queryWrapperTwo);
                String tableNameTwo = taskInputTwo.getTableName();
                String tableAsNameTwo = taskInputTwo.getTableAsName();
                joint += " " + tableName + " " + " " + tableAsName + " " + nodeJoint.getJoint()
                        + " " + tableNameTwo + " " + " " + tableAsNameTwo + " on " + nodeJoint.getOneFie()
                        + " = " + nodeJoint.getTwoFie();

            }
        } else {
            QueryWrapper<TaskInput> wrapper = new QueryWrapper<>();
            wrapper.eq("task_id", taskId);
            TaskInput taskInput = taskInputService.getOne(wrapper);
            String tableName = taskInput.getTableName();
            String tableAsName = taskInput.getTableAsName();
            joint = " " + tableName + " " + tableAsName;
        }
        return joint;
    }


    @NotNull
    private void getString(Long pageNum,
                           String fieName,
                           String joint,
                           Long basicId,
                           Long newBasicId,
                           Long tableId,
                           HashMap<String, String> map,
                           Long one,
                           Integer two) {
        String sqlSelect = " SELECT " + fieName + " FROM " + joint + " LIMIT " + PAGE_SIZE + " OFFSET  " + pageNum;
        log.info(sqlSelect);

        //log.info("执行{}查询的方法",sqlSelect);
        Result<DataValue[][]> tableValueResult = datasourceFeign.findTableValueToArray(basicId, sqlSelect, one, two);
        log.info(tableValueResult);

        DataValue[][] data = tableValueResult.getData();
        log.info("执行{}查询的方法结束", sqlSelect);
//        for (DataValue[] datum : data) {
//            for (DataValue dataValue : datum) {
//                String key = dataValue.getKey();
//                String newKey = map.get(key);
//                dataValue.setKey(newKey);
//            }
//        }
//        Result result = datasourceFeign.addProduct(newBasicId, tableId, data);
//        log.info("{}添加结束", result);
        executeTheRule(data,map,newBasicId,tableId);

    }


    private void executeTheRule(DataValue[][] dataValues,HashMap<String,String> map, Long newBasicId,
                                Long tableId) {
        // 创建一个单线程的ExecutorService
        ExecutorService executor = Executors.newSingleThreadExecutor();

        // 创建一个链表来保存任务
        LinkedList<Callable<DataValue[][]>> tasks = new LinkedList<>();

        // 初始化第一个任务
        tasks.add(() -> {
            log.info(dataValues);
            return dataValues;
        });

        // 提交第一个任务
        Future<DataValue[][]> currentFuture = executor.submit(tasks.poll());
        for (int i = 1; i <= 4; i++) {
            final Future<DataValue[][]> finalCurrentFuture = currentFuture;
            log.info(finalCurrentFuture);
            Callable<DataValue[][]> task = () -> {

                DataValue[][] prevResult = finalCurrentFuture.get();

                return prevResult;
            };

            // 提交任务并更新当前Future
            currentFuture = executor.submit(task);

            // 等待当前任务完成
            try {
                currentFuture.get();
                System.out.println("Task " + i + " completed with result:");
            } catch (InterruptedException | ExecutionException e) {
                Thread.currentThread().interrupt();
                System.out.println("Task execution failed: " + e.getMessage());
                break;
            }
        }
        try {
            DataValue[][] afterFilteringDataValue = currentFuture.get();
            for (DataValue[] datum : afterFilteringDataValue) {
                for (DataValue dataValue : datum) {
                    String key = dataValue.getKey();
                    String newKey = map.get(key);
                    dataValue.setKey(newKey);
                }
            }
            Result result = datasourceFeign.addProduct(newBasicId, tableId, afterFilteringDataValue);
            log.info("{}添加结束", result);

        } catch (InterruptedException e) {
            throw new RuntimeException(e);
        } catch (ExecutionException e) {
            throw new RuntimeException(e);
        }

        // 关闭ExecutorService
        executor.shutdown();
    }

}
